@@ -24,7 +24,8 @@ except Exception as err:
 
 from openmdao.core.constants import INT_DTYPE, _DEFAULT_REPORTS_DIR, _ReprClass
 from openmdao.core.analysis_error import AnalysisError
-from openmdao.core.driver import Driver, RecordingDebugging
+from openmdao.core.driver import Driver, RecordingDebugging, filter_by_meta
+from openmdao.core.group import Group
 from openmdao.utils.class_util import WeakMethodWrapper
 from openmdao.utils.mpi import FakeComm, MPI
 from openmdao.utils.om_warnings import issue_warning, warn_deprecation
@@ -173,15 +174,17 @@ class pyOptSparseDriver(Driver):
     _in_user_function :bool
         This is set to True at the start of a pyoptsparse callback to _objfunc and _gradfunc, and
         restored to False at the finish of each callback.
-    _indep_list : list
-        List of design variables.
-    _quantities : list
+    _nl_responses : list
         Contains the objectives plus nonlinear constraints.
     _signal_cache : <Function>
         Cached function pointer that was assigned as handler for signal defined in option
         user_terminate_signal.
+    _total_jac_sparsity : dict, str, or None
+        Specifies sparsity of sub-jacobians of the total jacobian.
     _user_termination_flag : bool
         This is set to True when the user sends a signal to terminate the job.
+    _model_ran : bool
+        This is set to True after the full model has been run at least once.
     """
 
     def __init__(self, **kwargs):
@@ -205,6 +208,7 @@ class pyOptSparseDriver(Driver):
         self.supports['multiple_objectives'] = True
         self.supports['two_sided_constraints'] = True
         self.supports['linear_constraints'] = True
+        self.supports['linear_only_designvars'] = True
         self.supports['simultaneous_derivatives'] = True
         self.supports['total_jac_sparsity'] = True
 
@@ -224,8 +228,7 @@ class pyOptSparseDriver(Driver):
         # the fail flag (e.g. IPOPT) to recognize a bad point and respond accordingly
         self._fill_NANs = False
 
-        self._indep_list = []
-        self._quantities = []
+        self._nl_responses = []
         self.fail = False
         self._signal_cache = None
         self._user_termination_flag = False
@@ -233,6 +236,8 @@ class pyOptSparseDriver(Driver):
         self._check_jac = False
         self._exc_info = None
         self._total_jac_format = 'dict'
+        self._total_jac_sparsity = None
+        self._model_ran = False
 
         self.cite = CITATIONS
 
@@ -246,7 +251,7 @@ class pyOptSparseDriver(Driver):
                              desc='Title of this optimization run')
         self.options.declare('print_opt_prob', types=bool, default=False,
                              desc='Print the opt problem summary before running the optimization')
-        self.options.declare('print_results', types=bool, default=True,
+        self.options.declare('print_results', types=(bool, str), default=True,
                              desc='Print pyOpt results if True')
         self.options.declare('gradient_method', default='openmdao',
                              values={'openmdao', 'pyopt_fd', 'snopt_fd'},
@@ -299,7 +304,7 @@ class pyOptSparseDriver(Driver):
         """
         warn_deprecation("The 'hotstart_file' attribute is deprecated. "
                          "Use the 'hotstart_file' option instead.")
-        return self.options['hist_file']
+        return self.options['hotstart_file']
 
     @hotstart_file.setter
     def hotstart_file(self, file_name):
@@ -332,30 +337,9 @@ class pyOptSparseDriver(Driver):
                                ' but the selected optimizer ({0}) does not support'
                                ' multiple objectives.'.format(self.options['optimizer']))
 
+        self._model_ran = False
         self._setup_tot_jac_sparsity()
 
-    def get_driver_objective_calls(self):
-        """
-        Return number of objective evaluations made during a driver run.
-
-        Returns
-        -------
-        int
-            Number of objective evaluations made during a driver run.
-        """
-        return self.pyopt_solution.userObjCalls if self.pyopt_solution else None
-
-    def get_driver_derivative_calls(self):
-        """
-        Return number of derivative evaluations made during a driver run.
-
-        Returns
-        -------
-        int
-            Number of derivative evaluations made during a driver run.
-        """
-        return self.pyopt_solution.userSensCalls if self.pyopt_solution else None
-
     def run(self):
         """
         Excute pyOptsparse.
@@ -368,15 +352,16 @@ class pyOptSparseDriver(Driver):
         bool
             Failure flag; True if failed to converge, False is successful.
         """
+        self.result.reset()
         problem = self._problem()
         model = problem.model
-        relevant = model._relevant
+        relevance = model._relevance
 
         self.pyopt_solution = None
         self._total_jac = None
+        self._total_jac_linear = None
         self.iter_count = 0
-        fwd = problem._mode == 'fwd'
-        self._quantities = []
+        self._nl_responses = []
 
         optimizer = self.options['optimizer']
 
@@ -386,43 +371,37 @@ class pyOptSparseDriver(Driver):
         self._check_for_invalid_desvar_values()
         self._check_jac = self.options['singular_jac_behavior'] in ['error', 'warn']
 
+        linear_constraints = [key for key, con in self._cons.items() if con['linear']]
+
         # Only need initial run if we have linear constraints or if we are using an optimizer that
         # doesn't perform one initially.
         model_ran = False
-        if optimizer in run_required or np.any([con['linear'] for con in self._cons.values()]):
+        if optimizer in run_required or linear_constraints:
             with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
-                # Initial Run
-                model.run_solve_nonlinear()
+                self._run_solve_nonlinear()
                 rec.abs = 0.0
                 rec.rel = 0.0
                 model_ran = True
             self.iter_count += 1
 
-        # compute dynamic simul deriv coloring
-        self._get_coloring(run_model=not model_ran)
+        self._model_ran = model_ran
+        self._coloring_info.run_model = not model_ran
 
         comm = None if isinstance(problem.comm, FakeComm) else problem.comm
         opt_prob = Optimization(self.options['title'], WeakMethodWrapper(self, '_objfunc'),
                                 comm=comm)
 
-        # Add all design variables
-        self._indep_list = indep_list = list(self._designvars)
-        self._indep_list_prom = indep_list_prom = []
-
         input_vals = self.get_design_var_values()
 
         for name, meta in self._designvars.items():
             # translate absolute var names to promoted names for pyoptsparse
-            prom_name = model._get_prom_name(name)
-            indep_list_prom.append(prom_name)
-
             size = meta['global_size'] if meta['distributed'] else meta['size']
             if pyoptsparse_version is None or pyoptsparse_version < Version('2.6.1'):
-                opt_prob.addVarGroup(prom_name, size, type='c',
+                opt_prob.addVarGroup(name, size, type='c',
                                      value=input_vals[name],
                                      lower=meta['lower'], upper=meta['upper'])
             else:
-                opt_prob.addVarGroup(prom_name, size, varType='c',
+                opt_prob.addVarGroup(name, size, varType='c',
                                      value=input_vals[name],
                                      lower=meta['lower'], upper=meta['upper'])
 
@@ -435,14 +414,11 @@ class pyOptSparseDriver(Driver):
         objs = self.get_objective_values()
         for name in objs:
             opt_prob.addObj(model._get_prom_name(name))
-            self._quantities.append(name)
-
-        cons_to_remove = set()
+            self._nl_responses.append(name)
 
         # Calculate and save derivatives for any linear constraints.
-        lcons = [key for (key, con) in self._cons.items() if con['linear']]
-        if len(lcons) > 0:
-            _lin_jacs = self._compute_totals(of=lcons, wrt=indep_list,
+        if linear_constraints:
+            _lin_jacs = self._compute_totals(of=linear_constraints, wrt=list(self._lin_dvs),
                                              return_format=self._total_jac_format)
             _con_vals = self.get_constraint_values(lintype='linear')
             # convert all of our linear constraint jacs to COO format. Otherwise pyoptsparse will
@@ -463,101 +439,85 @@ class pyOptSparseDriver(Driver):
                             # by pyoptsparse.
                             jacdct[n] = {'coo': [mat.row, mat.col, mat.data], 'shape': mat.shape}
 
-        # Add all equality constraints
-        for name, meta in self._cons.items():
-            if meta['equals'] is None:
-                continue
-            size = meta['global_size'] if meta['distributed'] else meta['size']
-            lower = upper = meta['equals']
-            path = meta['source']
-            if fwd:
-                wrt = [v for v in indep_list if path in relevant[self._designvars[v]['source']]]
-            else:
-                rels = relevant[path]
-                wrt = [v for v in indep_list if self._designvars[v]['source'] in rels]
+        # # compute dynamic simul deriv coloring
+        problem.get_total_coloring(self._coloring_info, run_model=not model_ran)
 
-            prom_name = model._get_prom_name(name)
+        bad_resps = [n for n in relevance._no_dv_responses if n in self._cons]
+        bad_cons = [n for n, m in self._cons.items() if m['source'] in bad_resps]
 
-            if not wrt:
-                issue_warning(f"Equality constraint '{prom_name}' does not depend on any design "
-                              "variables and was not added to the optimization.")
-                cons_to_remove.add(name)
-                continue
+        if bad_cons:
+            issue_warning(f"Equality constraint(s) {sorted(bad_cons)} do not depend on any design "
+                          "variables and were not added to the optimization.")
 
-            # convert wrt to use promoted names
-            wrt_prom = model._prom_names_list(wrt)
+            for name in bad_cons:
+                del self._cons[name]
+                del self._responses[name]
 
-            if meta['linear']:
-                jac = {w: _lin_jacs[name][w] for w in wrt}
-                jac_prom = model._prom_names_dict(jac)
-                opt_prob.addConGroup(prom_name, size,
-                                     lower=lower - _y_intercepts[name],
-                                     upper=upper - _y_intercepts[name],
-                                     linear=True, wrt=wrt_prom, jac=jac_prom)
-            else:
-                if name in self._res_subjacs:
-                    resjac = self._res_subjacs[name]
-                    jac = {n: resjac[self._designvars[n]['source']] for n in wrt}
-                    jac_prom = model._prom_names_jac(jac)
-                else:
-                    jac = None
-                    jac_prom = None
+        eqcons = {n: m for n, m in self._cons.items() if m['equals'] is not None}
+        if eqcons:
+            # Add all equality constraints
+            for name, meta in eqcons.items():
+                size = meta['global_size'] if meta['distributed'] else meta['size']
+                lower = upper = meta['equals']
 
-                opt_prob.addConGroup(prom_name, size, lower=lower, upper=upper,
-                                     wrt=wrt_prom, jac=jac_prom)
-                self._quantities.append(name)
+                # set equality constraints as reverse seeds to see what dvs are relevant
+                with relevance.seeds_active(rev_seeds=meta['source']):
 
-        # Add all inequality constraints
-        for name, meta in self._cons.items():
-            if meta['equals'] is not None:
-                continue
-            size = meta['global_size'] if meta['distributed'] else meta['size']
+                    if meta['linear']:
+                        wrts = [v for v in self._lin_dvs
+                                if relevance.is_relevant(self._lin_dvs[v]['source'])]
+                        jac = {w: _lin_jacs[name][w] for w in wrts}
+                        opt_prob.addConGroup(name, size,
+                                             lower=lower - _y_intercepts[name],
+                                             upper=upper - _y_intercepts[name],
+                                             linear=True, wrt=wrts, jac=jac)
+                    else:
+                        wrts = [v for v in self._nl_dvs
+                                if relevance.is_relevant(self._nl_dvs[v]['source'])]
 
-            # Bounds - double sided is supported
-            lower = meta['lower']
-            upper = meta['upper']
+                        if name in self._con_subjacs:
+                            resjac = self._con_subjacs[name]
+                            jac = {n: resjac[n] for n in wrts}
+                        else:
+                            jac = None
 
-            path = meta['source']
+                        opt_prob.addConGroup(name, size, lower=lower, upper=upper, wrt=wrts,
+                                             jac=jac)
+                        self._nl_responses.append(name)
 
-            if fwd:
-                wrt = [v for v in indep_list if path in relevant[self._designvars[v]['source']]]
-            else:
-                rels = relevant[path]
-                wrt = [v for v in indep_list if self._designvars[v]['source'] in rels]
+        ineqcons = {n: m for n, m in self._cons.items() if m['equals'] is None}
+        if ineqcons:
+            # Add all inequality constraints
+            for name, meta in ineqcons.items():
+                size = meta['global_size'] if meta['distributed'] else meta['size']
 
-            prom_name = model._get_prom_name(name)
+                # Bounds - double sided is supported
+                lower = meta['lower']
+                upper = meta['upper']
 
-            if not wrt:
-                issue_warning(f"Inequality constraint '{prom_name}' does not depend on any design "
-                              "variables and was not added to the optimization.")
-                cons_to_remove.add(name)
-                continue
+                # set inequality constraints as reverse seeds to see what dvs are relevant
+                with relevance.seeds_active(rev_seeds=(meta['source'],)):
 
-            # convert wrt to use promoted names
-            wrt_prom = model._prom_names_list(wrt)
+                    if meta['linear']:
+                        wrts = [v for v in self._lin_dvs
+                                if relevance.is_relevant(self._lin_dvs[v]['source'])]
+                        jac = {w: _lin_jacs[name][w] for w in wrts}
+                        opt_prob.addConGroup(name, size,
+                                             upper=upper - _y_intercepts[name],
+                                             lower=lower - _y_intercepts[name],
+                                             linear=True, wrt=wrts, jac=jac)
+                    else:
+                        wrts = [v for v in self._nl_dvs
+                                if relevance.is_relevant(self._nl_dvs[v]['source'])]
+                        if name in self._con_subjacs:
+                            resjac = self._con_subjacs[name]
+                            jac = {n: resjac[n] for n in wrts}
+                        else:
+                            jac = None
 
-            if meta['linear']:
-                jac = {w: _lin_jacs[name][w] for w in wrt}
-                jac_prom = model._prom_names_dict(jac)
-                opt_prob.addConGroup(prom_name, size,
-                                     upper=upper - _y_intercepts[name],
-                                     lower=lower - _y_intercepts[name],
-                                     linear=True, wrt=wrt_prom, jac=jac_prom)
-            else:
-                if name in self._res_subjacs:
-                    resjac = self._res_subjacs[name]
-                    jac = {n: resjac[self._designvars[n]['source']] for n in wrt}
-                    jac_prom = model._prom_names_jac(jac)
-                else:
-                    jac = None
-                    jac_prom = None
-                opt_prob.addConGroup(prom_name, size, upper=upper, lower=lower,
-                                     wrt=wrt_prom, jac=jac_prom)
-                self._quantities.append(name)
-
-        for name in cons_to_remove:
-            del self._cons[name]
-            del self._responses[name]
+                        opt_prob.addConGroup(name, size, upper=upper, lower=lower,
+                                             wrt=wrts, jac=jac)
+                        self._nl_responses.append(name)
 
         # Instantiate the requested optimizer
         try:
@@ -661,17 +621,24 @@ class pyOptSparseDriver(Driver):
         # Print results
         if self.options['print_results']:
             if not MPI or model.comm.rank == 0:
-                print(sol)
+                if self.options['print_results'] == 'minimal':
+                    if hasattr(sol, 'summary_str'):
+                        print(sol.summary_str(minimal_print=True))
+                    else:
+                        print('minimal_print is not available for this solution')
+                        print(sol)
+                else:
+                    print(sol)
 
         # Pull optimal parameters back into framework and re-run, so that
         # framework is left in the right final state
         dv_dict = sol.getDVs()
-        for name in indep_list:
+        for name in self._designvars:
             self.set_design_var(name, dv_dict[model._get_prom_name(name)])
 
         with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
             try:
-                model.run_solve_nonlinear()
+                self._run_solve_nonlinear()
             except AnalysisError:
                 model._clear_iprint()
 
@@ -690,8 +657,10 @@ class pyOptSparseDriver(Driver):
             if optimizer == 'IPOPT':
                 if exit_status not in {0, 1}:
                     self.fail = True
-            elif exit_status > 2:
-                self.fail = True
+            else:
+                # exit status may be the empty string for optimizers that don't support it
+                if exit_status and exit_status > 2:
+                    self.fail = True
 
         except KeyError:
             # optimizers other than pySNOPT may not populate this dict
@@ -737,7 +706,7 @@ class pyOptSparseDriver(Driver):
             signal.signal(sigusr, self._signal_handler)
 
         try:
-            for name in self._indep_list:
+            for name in self._nl_dvs:
                 self.set_design_var(name, dv_dict[model._get_prom_name(name)])
 
             # print("Setting DV")
@@ -756,7 +725,11 @@ class pyOptSparseDriver(Driver):
                 self.iter_count += 1
                 try:
                     self._in_user_function = True
-                    model.run_solve_nonlinear()
+                    # deactivate the relevance if we haven't run the full model yet, so that
+                    # the full model will run at least once.
+                    with model._relevance.nonlinear_active('iter', active=self._model_ran):
+                        self._run_solve_nonlinear()
+                        self._model_ran = True
 
                 # Let the optimizer try to handle the error
                 except AnalysisError:
@@ -783,7 +756,7 @@ class pyOptSparseDriver(Driver):
 
         if fail > 0 and self._fill_NANs:
             for name in func_dict:
-                func_dict[name].fill(np.NAN)
+                func_dict[name].fill(np.nan)
 
         # convert func_dict to use promoted names
         func_dict = model._prom_names_dict(func_dict)
@@ -832,15 +805,19 @@ class pyOptSparseDriver(Driver):
 
             try:
                 self._in_user_function = True
-                sens_dict = self._compute_totals(of=self._quantities,
-                                                 wrt=self._indep_list,
+                sens_dict = self._compute_totals(of=self._nl_responses,
+                                                 wrt=self._nl_dvs,
                                                  return_format=self._total_jac_format)
 
                 # First time through, check for zero row/col.
                 if self._check_jac and self._total_jac is not None:
-                    raise_error = self.options['singular_jac_behavior'] == 'error'
-                    self._total_jac.check_total_jac(raise_error=raise_error,
-                                                    tol=self.options['singular_jac_tol'])
+                    for subsys in model.system_iter(include_self=True, recurse=True, typ=Group):
+                        if subsys._has_approx:
+                            break
+                    else:
+                        raise_error = self.options['singular_jac_behavior'] == 'error'
+                        self._total_jac.check_total_jac(raise_error=raise_error,
+                                                        tol=self.options['singular_jac_tol'])
                     self._check_jac = False
 
             # Let the optimizer try to handle the error
@@ -858,15 +835,14 @@ class pyOptSparseDriver(Driver):
                 # conversion of our dense array into a fully dense 'coo', which is bad.
                 # TODO: look into getting rid of all of these conversions!
                 new_sens = {}
-                res_subjacs = self._res_subjacs
+                con_subjacs = self._con_subjacs
 
-                for okey in self._quantities:
+                for okey in self._nl_responses:
                     new_sens[okey] = newdv = {}
-                    for ikey in self._designvars.keys():
-                        ikey_src = self._designvars[ikey]['source']
-                        if okey in res_subjacs and ikey_src in res_subjacs[okey]:
+                    for ikey in self._nl_dvs.keys():
+                        if okey in con_subjacs and ikey in con_subjacs[okey]:
                             arr = sens_dict[okey][ikey]
-                            coo = res_subjacs[okey][ikey_src]
+                            coo = con_subjacs[okey][ikey]
                             row, col, _ = coo['coo']
                             coo['coo'][2] = arr[row, col].flatten()
                             newdv[ikey] = coo
@@ -882,7 +858,7 @@ class pyOptSparseDriver(Driver):
         if fail > 0:
             # We need to cobble together a sens_dict of the correct size.
             # Best we can do is return zeros or NaNs.
-            for okey in self._quantities:
+            for okey in self._nl_responses:
                 if okey not in sens_dict:
                     sens_dict[okey] = {}
                 oval = func_dict[model._get_prom_name(okey)]
@@ -893,7 +869,7 @@ class pyOptSparseDriver(Driver):
                     if ikey not in sens_dict[okey] or self._fill_NANs:
                         sens_dict[okey][ikey] = np.zeros((osize, isize))
                         if self._fill_NANs:
-                            sens_dict[okey][ikey].fill(np.NAN)
+                            sens_dict[okey][ikey].fill(np.nan)
 
         # convert sens_dict to use promoted names
         sens_dict = model._prom_names_jac(sens_dict)
@@ -950,13 +926,13 @@ class pyOptSparseDriver(Driver):
             Current coloring.
         """
         total_sparsity = None
-        self._res_subjacs = {}
+        self._con_subjacs = {}
         coloring = coloring if coloring is not None else self._get_static_coloring()
         if coloring is not None:
             total_sparsity = coloring.get_subjac_sparsity()
             if self._total_jac_sparsity is not None:
                 raise RuntimeError("Total jac sparsity was set in both _total_coloring"
-                                   " and _total_jac_sparsity.")
+                                   " and _setup_tot_jac_sparsity.")
         elif self._total_jac_sparsity is not None:
             if isinstance(self._total_jac_sparsity, str):
                 with open(self._total_jac_sparsity, 'r') as f:
@@ -966,15 +942,19 @@ class pyOptSparseDriver(Driver):
         if total_sparsity is None:
             return
 
-        for res, dvdict in total_sparsity.items():  # res are 'driver' names (prom name or alias)
-            if res in self._objs:  # skip objectives
-                continue
-            self._res_subjacs[res] = {}
-            for dv, (rows, cols, shape) in dvdict.items():  # dvs are src names
-                rows = np.array(rows, dtype=INT_DTYPE)
-                cols = np.array(cols, dtype=INT_DTYPE)
+        use_approx = self._problem().model._owns_approx_of is not None
 
-                self._res_subjacs[res][dv] = {
+        # exclude linear cons and dvs that only impact linear cons
+        for con, conmeta in filter_by_meta(self._cons.items(), 'linear', exclude=True):
+            self._con_subjacs[con] = {}
+            consrc = conmeta['source']
+            for dv, dvmeta in self._nl_dvs.items():
+                if use_approx:
+                    dvsrc = dvmeta['source']
+                    rows, cols, shape = total_sparsity[consrc][dvsrc]
+                else:
+                    rows, cols, shape = total_sparsity[con][dv]
+                self._con_subjacs[con][dv] = {
                     'coo': [rows, cols, np.zeros(rows.size)],
                     'shape': shape,
                 }
